==> dart_icalendar-1.0.1.tar.gz: lib/models/todo_model.dart <==
import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:intl/intl.dart';

import 'package:dart_icalendar/models/attachment_model.dart';
import 'package:dart_icalendar/models/person_model.dart';
import 'package:dart_icalendar/models/recurrence_model.dart';

class Todo {
  final String uid;
  final String? summary;
  final String? description;
  final List<Attachment>? attachments;
  final List<Attendee>? attendees;
  final Organizer? organizer;
  final DateTime? created;
  final DateTime? lastModified;
  final DateTime? startDate;
  final DateTime? dueDate;
  final DateTime? completedDate;
  final RecurrenceRule? recurrenceRule;

  Todo({
    required this.uid,
    this.summary,
    this.description,
    this.attachments,
    this.attendees,
    this.organizer,
    this.created,
    this.lastModified,
    this.startDate,
    this.dueDate,
    this.completedDate,
    this.recurrenceRule,
  });

  bool isCompleted() {
    return completedDate != null;
  }

  String toICalString() {
    final buffer = StringBuffer();
    buffer.writeln('BEGIN:VTODO');
    buffer.writeln('UID:$uid');
    if (summary != null) {
      buffer.writeln('SUMMARY:$summary');
    }
    if (description != null) {
      buffer.writeln('DESCRIPTION:$description');
    }
    if (attachments != null && attachments!.isNotEmpty) {
      for (final attachment in attachments!) {
        buffer.writeln(attachment.toICalString());
      }
    }
    if (attendees != null && attendees!.isNotEmpty) {
      for (final attendee in attendees!) {
        buffer.writeln(attendee.toICalString());
      }
    }
    if (organizer != null) {
      buffer.writeln(organizer!.toICalString());
    }
    if (created != null) {
      buffer.writeln(
          'CREATED:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(created!.toUtc())}');
    }
    if (lastModified != null) {
      buffer.writeln(
          'LAST-MODIFIED:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(lastModified!.toUtc())}');
    }
    if (startDate != null) {
      buffer.writeln(
          'DTSTART:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(startDate!.toUtc())}');
    }
    if (dueDate != null) {
      buffer.writeln(
          'DUE:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(dueDate!.toUtc())}');
    }
    if (completedDate != null) {
      buffer.writeln(
          'COMPLETED:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(completedDate!.toUtc())}');
    }
    if (recurrenceRule != null) {
      buffer.writeln(recurrenceRule!.toICalString());
    }
    buffer.writeln('END:VTODO');
    return buffer.toString();
  }

  Todo copyWith({
    String? uid,
    String? summary,
    String? description,
    List<Attachment>? attachments,
    List<Attendee>? attendees,
    Organizer? organizer,
    DateTime? created,
    DateTime? lastModified,
    DateTime? startDate,
    DateTime? dueDate,
    DateTime? completedDate,
    RecurrenceRule? recurrenceRule,
  }) {
    return Todo(
      uid: uid ?? this.uid,
      summary: summary ?? this.summary,
      description: description ?? this.description,
      attachments: attachments ?? this.attachments,
      attendees: attendees ?? this.attendees,
      organizer: organizer ?? this.organizer,
      created: created ?? this.created,
      lastModified: lastModified ?? this.lastModified,
      startDate: startDate ?? this.startDate,
      dueDate: dueDate ?? this.dueDate,
      completedDate: completedDate ?? this.completedDate,
      recurrenceRule: recurrenceRule ?? this.recurrenceRule,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'uid': uid,
      'summary': summary,
      'description': description,
      'attachments': attachments?.map((x) => x.toMap()).toList(),
      'attendees': attendees?.map((x) => x.toMap()).toList(),
      'organizer': organizer?.toMap(),
      'created': created?.millisecondsSinceEpoch,
      'lastModified': lastModified?.millisecondsSinceEpoch,
      'startDate': startDate?.millisecondsSinceEpoch,
      'dueDate': dueDate?.millisecondsSinceEpoch,
      'completedDate': completedDate?.millisecondsSinceEpoch,
      'recurrenceRule': recurrenceRule?.toMap(),
    };
  }

  factory Todo.fromMap(Map<String, dynamic> map) {
    return Todo(
      uid: map['uid'] ?? '',
      summary: map['summary'],
      description: map['description'],
      attachments: map['attachments'] != null
          ? List<Attachment>.from(
              map['attachments']?.map((x) => Attachment.fromMap(x)))
          : null,
      attendees: map['attendees'] != null
          ? List<Attendee>.from(
              map['attendees']?.map((x) => Attendee.fromMap(x)))
          : null,
      organizer:
          map['organizer'] != null ? Organizer.fromMap(map['organizer']) : null,
      created: map['created'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['created'])
          : null,
      lastModified: map['lastModified'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['lastModified'])
          : null,
      startDate: map['startDate'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['startDate'])
          : null,
      dueDate: map['dueDate'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['dueDate'])
          : null,
      completedDate: map['completedDate'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['completedDate'])
          : null,
      recurrenceRule: map['recurrenceRule'] != null
          ? RecurrenceRule.fromMap(map['recurrenceRule'])
          : null,
    );
  }

  String toJson() => json.encode(toMap());

  factory Todo.fromJson(String source) => Todo.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Todo(uid: $uid, summary: $summary, description: $description, attachments: $attachments, attendees: $attendees, organizer: $organizer, created: $created, lastModified: $lastModified, startDate: $startDate, dueDate: $dueDate, completedDate: $completedDate, recurrenceRule: $recurrenceRule)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is Todo &&
        other.uid == uid &&
        other.summary == summary &&
        other.description == description &&
        listEquals(other.attachments, attachments) &&
        listEquals(other.attendees, attendees) &&
        other.organizer == organizer &&
        other.created == created &&
        other.lastModified == lastModified &&
        other.startDate == startDate &&
        other.dueDate == dueDate &&
        other.completedDate == completedDate &&
        other.recurrenceRule == recurrenceRule;
  }

  @override
  int get hashCode {
    return uid.hashCode ^
        summary.hashCode ^
        description.hashCode ^
        attachments.hashCode ^
        attendees.hashCode ^
        organizer.hashCode ^
        created.hashCode ^
        lastModified.hashCode ^
        startDate.hashCode ^
        dueDate.hashCode ^
        completedDate.hashCode ^
        recurrenceRule.hashCode;
  }
}

==> dart_icalendar-1.0.2.tar.gz: lib/models/todo_model.dart <==
import 'dart:convert';

import 'package:collection/collection.dart';
import 'package:intl/intl.dart';

import 'package:dart_icalendar/models/attachment_model.dart';
import 'package:dart_icalendar/models/person_model.dart';
import 'package:dart_icalendar/models/recurrence_model.dart';

class Todo {
  final String uid;
  final String? summary;
  final String? description;
  final List<Attachment>? attachments;
  final List<Attendee>? attendees;
  final Organizer? organizer;
  final DateTime? created;
  final DateTime? lastModified;
  final DateTime? startDate;
  final DateTime? dueDate;
  final DateTime? completedDate;
  final RecurrenceRule? recurrenceRule;

  Todo({
    required this.uid,
    this.summary,
    this.description,
    this.attachments,
    this.attendees,
    this.organizer,
    this.created,
    this.lastModified,
    this.startDate,
    this.dueDate,
    this.completedDate,
    this.recurrenceRule,
  });

  bool isCompleted() {
    return completedDate != null;
  }

  String toICalString() {
    final buffer = StringBuffer();
    buffer.writeln('BEGIN:VTODO');
    buffer.writeln('UID:$uid');
    if (summary != null) {
      buffer.writeln('SUMMARY:$summary');
    }
    if (description != null) {
      buffer.writeln('DESCRIPTION:$description');
    }
    if (attachments != null && attachments!.isNotEmpty) {
      for (final attachment in attachments!) {
        buffer.writeln(attachment.toICalString());
      }
    }
    if (attendees != null && attendees!.isNotEmpty) {
      for (final attendee in attendees!) {
        buffer.writeln(attendee.toICalString());
      }
    }
    if (organizer != null) {
      buffer.writeln(organizer!.toICalString());
    }
    if (created != null) {
      buffer.writeln(
          'CREATED:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(created!.toUtc())}');
    }
    if (lastModified != null) {
      buffer.writeln(
          'LAST-MODIFIED:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(lastModified!.toUtc())}');
    }
    if (startDate != null) {
      buffer.writeln(
          'DTSTART:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(startDate!.toUtc())}');
    }
    if (dueDate != null) {
      buffer.writeln(
          'DUE:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(dueDate!.toUtc())}');
    }
    if (completedDate != null) {
      buffer.writeln(
          'COMPLETED:${DateFormat('yyyyMMdd\'T\'HHmmss\'Z\'').format(completedDate!.toUtc())}');
    }
    if (recurrenceRule != null) {
      buffer.writeln(recurrenceRule!.toICalString());
    }
    buffer.writeln('END:VTODO');
    return buffer.toString();
  }

  Todo copyWith({
    String? uid,
    String? summary,
    String? description,
    List<Attachment>? attachments,
    List<Attendee>? attendees,
    Organizer? organizer,
    DateTime? created,
    DateTime? lastModified,
    DateTime? startDate,
    DateTime? dueDate,
    DateTime? completedDate,
    RecurrenceRule? recurrenceRule,
  }) {
    return Todo(
      uid: uid ?? this.uid,
      summary: summary ?? this.summary,
      description: description ?? this.description,
      attachments: attachments ?? this.attachments,
      attendees: attendees ?? this.attendees,
      organizer: organizer ?? this.organizer,
      created: created ?? this.created,
      lastModified: lastModified ?? this.lastModified,
      startDate: startDate ?? this.startDate,
      dueDate: dueDate ?? this.dueDate,
      completedDate: completedDate ?? this.completedDate,
      recurrenceRule: recurrenceRule ?? this.recurrenceRule,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'uid': uid,
      'summary': summary,
      'description': description,
      'attachments': attachments?.map((x) => x.toMap()).toList(),
      'attendees': attendees?.map((x) => x.toMap()).toList(),
      'organizer': organizer?.toMap(),
      'created': created?.millisecondsSinceEpoch,
      'last_modified': lastModified?.millisecondsSinceEpoch,
      'start_date': startDate?.millisecondsSinceEpoch,
      'due_date': dueDate?.millisecondsSinceEpoch,
      'completed_date': completedDate?.millisecondsSinceEpoch,
      'recurrence_rule': recurrenceRule?.toMap(),
    };
  }

  factory Todo.fromMap(Map<String, dynamic> map) {
    return Todo(
      uid: map['uid'] ?? '',
      summary: map['summary'],
      description: map['description'],
      attachments: map['attachments'] != null
          ? List<Attachment>.from(
              map['attachments']?.map((x) => Attachment.fromMap(x)))
          : null,
      attendees: map['attendees'] != null
          ? List<Attendee>.from(
              map['attendees']?.map((x) => Attendee.fromMap(x)))
          : null,
      organizer:
          map['organizer'] != null ? Organizer.fromMap(map['organizer']) : null,
      created: map['created'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['created'])
          : null,
      lastModified: map['last_modified'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['last_modified'])
          : null,
      startDate: map['start_date'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['start_date'])
          : null,
      dueDate: map['due_date'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['due_date'])
          : null,
      completedDate: map['completed_date'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['completed_date'])
          : null,
      recurrenceRule: map['recurrence_rule'] != null
          ? RecurrenceRule.fromMap(map['recurrence_rule'])
          : null,
    );
  }

  String toJson() => json.encode(toMap());

  factory Todo.fromJson(String source) => Todo.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Todo(uid: $uid, summary: $summary, description: $description, attachments: $attachments, attendees: $attendees, organizer: $organizer, created: $created, lastModified: $lastModified, startDate: $startDate, dueDate: $dueDate, completedDate: $completedDate, recurrenceRule: $recurrenceRule)';
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;

    return other is Todo &&
        other.uid == uid &&
        other.summary == summary &&
        other.description == description &&
        listEquals(other.attachments, attachments) &&
        listEquals(other.attendees, attendees) &&
        other.organizer == organizer &&
        other.created == created &&
        other.lastModified == lastModified &&
        other.startDate == startDate &&
        other.dueDate == dueDate &&
        other.completedDate == completedDate &&
        other.recurrenceRule == recurrenceRule;
  }

  @override
  int get hashCode {
    return uid.hashCode ^
        summary.hashCode ^
        description.hashCode ^
        attachments.hashCode ^
        attendees.hashCode ^
        organizer.hashCode ^
        created.hashCode ^
        lastModified.hashCode ^
        startDate.hashCode ^
        dueDate.hashCode ^
        completedDate.hashCode ^
        recurrenceRule.hashCode;
  }
}
